/* Sergio Nuñez Covarrubias */

#ifndef ClientEventHandler_H
#define ClientEventHandler_H

#include "EventHandler.H"
#include "Random.H"
#include "Game.H"
#include "Movement.h"
#include "ModuleSet.h"
#include "Watcher.H"
#include "WatcherAux.H"
#include "History.H"
#include "SquadCombatAI.H"
#include "TerrainModule.H"
#include "TerrainAnalysis.H"
#include "Finance.H"
#include "Movement.H"
#include "SimpleTerrain.H"
#include "ST_ForceField.H"
#include "MineralCluster.H"
#include "Gather_accessories.H"
#include "SpecialScouting.H"
#include "SpecialWorker.H"
#include "SimpleGather.H"
#include "SpecialBaseCommander.H"
#include "SpecialDefenseCommander.H"
#include "Nets.H"
#include "NewBase.H"


/* Estructura para acceder al estado del juego */
struct ClientAIState {
	
	GameStateModule * gsm;
	
	/* 0 si modo grafico 3d desactivado, !=0 activado */
	GfxModule * gfxm;  
  
	/* true si hay que dibujar el gfx frame */
	bool just_drew;
  
	/* true si hay que salir del juego */
	bool quit;
  
	/* true si ha ocurrido un error */
	bool error;

	/* Constructor */	
	ClientAIState(GameStateModule * gsm_, GfxModule * gfxm_) :	gsm(gsm_), gfxm(gfxm_){		
		just_drew = quit = error = false;		
	}	
};


/* Clase ClientEventHandler */
class ClientEventHandler : public EventHandler {

public:
	
	/* Estructura ClienteAIState */
	ClientAIState &state;
	
	/* Referencia a un conjunto de modulos de apoyo */
	ModuleSet & ms;
  
	/* Object Scouting */
	SpecialScouting* scouting;
	
	/* Object SimpleGather */
	SimpleGather* simpleGather;
		
	/* Vector de bases */
	Vector<SpecialBaseCommander*> bases;
  
	/* Number Tiles Known */
	short nTilesKnown;
  	
	/* Variable para indicar si ya se ha realizado la inicializacion */
	bool init;
  
	/* Variable que indica si se necesita realizar la actividad gather. Siempre true */
	bool needGather;
  
	/* Vector de workers */
	Vector<GameObj*> workers;
	
	/* Vector de marines */
	Vector<GameObj*> marines;
	
	/* Vector de tanques */
	Vector<GameObj*> tanks;
  
	/* Vector de Control Centers */
	Vector <GameObj *> controlCenters;
	
	/* Numero maximo de unidades que pueden realizar de forma simultanea la actividad scouting */
	static const unsigned NUMBER_OF_SCOUTS = 2;
  
	/* Constructor */
	ClientEventHandler(ClientAIState &state_, sint4 seed, ModuleSet & msGeneral, ModuleSet &msGather):
	 state(state_), ms(msGather), fm(msGather.get_finance()){
		rand.srand(seed);
		nTilesKnown = 0;		
		init = false;
		needGather = true;
			
		/* Crea el objeto Scouting */
		scouting = new SpecialScouting(seed, msGeneral, msGather);
		
		/* Crea e inicializa el objeto SimpleGather */
		simpleGather = new SimpleGather(state.gsm, seed, &workers, msGather, &controlCenters);
		simpleGather->init();
		
		/* Crea el objeto de redes bayesianas */
		nets = new Nets();
		
		/* Inicializa el resto de varibels */
		attack1 = NULL;
		attack2 = NULL;
		
		newBase = NULL;
		nControlCenters = 1;		
		buildCC = false;
	}
  
	/* Destructor */
	~ClientEventHandler(){
		delete scouting;
		delete simpleGather;
		delete nets;		
	}

	/* Propotipo del metodo handle_event */
	bool handle_event(const Event &e);
    

private:

	/* Modulo de apoyo para las finanzas */
	Finance::Module & fm;
	
	/* Objeto de las redes bayesianas */
	Nets* nets;
	
	/* Objeto del ataque 1 */
	SpecialDefenseCommander * attack1;
	
	/* Objeto del ataque 2 */
	SpecialDefenseCommander * attack2;
	
	/* Conjunto de control centers enemigos conocidos */
	map<string, ScalarPoint> enemyControlCenters;	
	
	/* Conjunto de Barracks y Factory enemigos conocidos */
	map<string, ScalarPoint> otherEnemyBuildings;
	
	/* Iterador de tipo map */
	map<string, ScalarPoint>::iterator it;
	
	/* Numero de control centers */
	size_t nControlCenters;
	
	/* Vector que almacena las posiciones de las bases */
	Vector<sint4> posBase;
		
	bool buildCC;
		
	/* Prototipo del metodo compute_actions */
	void compute_actions();	
  
	/* Prototipo del metodo initGame */
	void initGame(const Game::ObjCont &objs);
	
	/* Prototipo del metodo asignarClusters */
	void asignarClusters(Vector<boost::shared_ptr<MineralCluster> > clusters, const Map<GameTile> &map);
	
	/* Prototipo del metodo rutaAccesible */
	bool rutaAccesible(TerrainBase::Loc locA, TerrainBase::Loc locB, const Map<GameTile> &map);
	
	/* Prototipo del metodo getTile */
	TerrainBase::Loc getTile(TerrainBase::Loc point);
	
	/* Prototipo del metodo getClustersNoAsignados */
	Vector<boost::shared_ptr<MineralCluster> > getClustersNoAsignados(Vector<boost::shared_ptr<MineralCluster> > mineralClusters);
	
	/* Prototipo del metodo getSmallDistanceToMinerals */
	float getSmallDistanceToMinerals(boost::shared_ptr<MineralCluster> cluster);
	
	/* Prototipo del metodo isPossible */
	float isPossible(boost::shared_ptr<MineralCluster> cluster);
	
	/* Prototipo del metodo defaultDefense */
	void defaultDefense(Vector<GameObj*> * vector);
	
	/* Prototipo del metodo createNewBase */
	void createNewBase(boost::shared_ptr<MineralCluster> cluster);
	
	/* Prototipo del metodo getLocationEnemyToNewBase */
	TerrainBase::Loc getLocationEnemyToNewBase(TerrainBase::Loc locCluster);
	
	/* Prototipo del metodo buildNewBase */
	void buildNewBase(const Map<GameTile> &map);
	
	/* Prototipo del metodo examineProblems */
	void examineProblems();
	
	/* Prototipo del metodo evaluateAttack */
	void evaluateAttack(SpecialDefenseCommander ** attack, bool weAreUnderAttack);
	
	/* Prototipo del metodo createAttack */
	void createAttack(std::string tipo);
	
	/* Prototipo del metodo getPosBase */
	sint4 getPosBase(TerrainBase::Loc locCC);
  
	/* Objeto random */
	Random rand;
	
	/* Objeto del procedimiento de creacion de una nueva base */
	NewBase * newBase;	
};

#endif