/* Sergio Nuñez Covarrubias */

#include "Nets.H"
#include <iostream.h>

using namespace std;

DSL_network networkAttack;
DSL_network networkDefense;
DSL_network networkBuildingBarracks;
DSL_network networkBuildingFactory;
DSL_network networkTrainingWorker;
DSL_network networkTrainingUnit;
DSL_network networkNewBase;

/*
 * Carga todas las redes bayesianas en memoria.
 */
void Nets::init(){

	networkAttack.ReadFile("./apps/BlackRose/src/nets/NetworkAttack.xdsl");	
	networkDefense.ReadFile("./apps/BlackRose/src/nets/NetworkDefense.xdsl");
	networkBuildingBarracks.ReadFile("./apps/BlackRose/src/nets/NetworkBuildingBarracks.xdsl");
	networkBuildingFactory.ReadFile("./apps/BlackRose/src/nets/NetworkBuildingFactory.xdsl");
	networkTrainingWorker.ReadFile("./apps/BlackRose/src/nets/NetworkTrainingWorker.xdsl");
	networkTrainingUnit.ReadFile("./apps/BlackRose/src/nets/NetworkTrainingUnit.xdsl");
	networkNewBase.ReadFile("./apps/BlackRose/src/nets/NetworkNewBase.xdsl");

}

/*
 * Ejecuta la red bayesiana de ataque con los parametros como valores de entrada.
 * @param force Poder militar.
 * @param localized Indica si se ha localizado alguna base enemiga.
 * @param finished Tiempo transcurrido de la partida.
 * @param attacking Indica si se esta realizando algun ataque.
 * @return Resultado obtenido al ejecutar la red bayesiana.
 */
float Nets::getValueAttack(float force, float localized, float finished, float attacking){
	
	DSL_doubleArray theProbs;	
	DSL_idArray * names;
	DSL_Dmatrix * value;
	
	/* Asignamos los nuevos valores de entrada */
	
	/* Nodo Force */
	int nforce = networkAttack.FindNode("force");	
	theProbs.SetSize(2);
	theProbs[0] = force;
	theProbs[1] = (1 - force);	
	networkAttack.GetNode(nforce)->Definition()->SetDefinition(theProbs);	
	
	/* Nodo Located enemy? */	
	int nlocated = networkAttack.FindNode("located");	
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = localized;
	theProbs[1] = (1 - localized);
	networkAttack.GetNode(nlocated)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Time */
	int ntime = networkAttack.FindNode("time");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = finished;
	theProbs[1] = (1 - finished);
	networkAttack.GetNode(ntime)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Attacking? */
	int nattacking = networkAttack.FindNode("attacking");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = attacking;
	theProbs[1] = (1 - attacking);
	networkAttack.GetNode(nattacking)->Definition()->SetDefinition(theProbs);
	
	
	/* Actualizamos la red */
	networkAttack.UpdateBeliefs();
	
	
	/* Nodo Attack */	
	int nattack = networkAttack.FindNode("attack");
	
	networkAttack.GetNode(nattack)->Value()->GetValue(&value);
	names = networkAttack.GetNode(nattack)->Definition()->GetOutcomesNames();
	
	return (*value)[0];
}

/*
 * Ejecuta la red bayesiana de defensa con los parametros como valores de entrada.
 * @param enemyAttack Indica si el enemigo esta atacando la base.
 * @param building Indica si la base cuenta con algun edificio.
 * @param longTime Indica el tiempo transcurrido desde el ultimo ataque recibido.
 * @param finished Tiempo transcurrido de la partida.
 * @param playerAttack Indica si se esta realizando algun ataque.
 * @return Resultado obtenido al ejecutar la red bayesiana.
 */
float Nets::getValueDefense(float enemyAttack, float building, float longTime, float finished, float playerAttack){
	
	DSL_doubleArray theProbs;
	DSL_idArray * names;
	DSL_Dmatrix * value;
	
	/* Asignamos los nuevos valores de entrada */
	
	/* Nodo Enemy attacking? */
	int nenemy = networkDefense.FindNode("enemy");	
	theProbs.SetSize(2);
	theProbs[0] = enemyAttack;
	theProbs[1] = (1 - enemyAttack);	
	networkDefense.GetNode(nenemy)->Definition()->SetDefinition(theProbs);	
	
	/* Nodo Is there building? */	
	int nbuilding = networkDefense.FindNode("building");	
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = building;
	theProbs[1] = (1 - building);
	networkDefense.GetNode(nbuilding)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Time last attack */
	int nlast = networkDefense.FindNode("last");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = longTime;
	theProbs[1] = (1 - longTime);
	networkDefense.GetNode(nlast)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Time */
	int ntime = networkDefense.FindNode("time");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = finished;
	theProbs[1] = (1 - finished);
	networkDefense.GetNode(ntime)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Are we attacking? */
	int nattacking = networkDefense.FindNode("attacking");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = playerAttack;
	theProbs[1] = (1 - playerAttack);
	networkDefense.GetNode(nattacking)->Definition()->SetDefinition(theProbs);
	
	/* Actualizamos la red */
	networkDefense.UpdateBeliefs();	
	
	/* Nodo Defense */	
	int ndefense = networkDefense.FindNode("defense");
	
	networkDefense.GetNode(ndefense)->Value()->GetValue(&value);
	names = networkDefense.GetNode(ndefense)->Definition()->GetOutcomesNames();
	
	return (*value)[0];
}

/*
 * Ejecuta la red bayesiana de construccion de barracks con los parametros como valores de entrada.
 * @param few Cantidad de minerales relativa a la construccion de edificios.
 * @param yesBarracks Indica si la base actual dispone del edifico barracks.
 * @param yesBuildingNow Indica si se esta construyendo el edificio barracks (en la base actual).
 * @param finished Tiempo transcurrido de la partida.
 * @param yesMoreBases Indica si se dispone de mas de una base.
 * @param small Distancia de la base (actual) al edificio enemigo mas cercano.
 * @param yesControlCenter Indica si al menos se cuenta con un edificio control center.
 * @param yesWorker Indica si se dispone de al menos una unidad worker.
 * @return Resultado obtenido al ejecutar la red bayesiana.
 */ 
float Nets::getValueBuildingBarracks(float few, float yesBarracks, float yesBuildingNow, float finished,
												   float yesMoreBases, float small, float yesControlCenter, float yesWorker){
	
	DSL_doubleArray theProbs;	
	DSL_idArray * names;
	DSL_Dmatrix * value;
	
	/* Asignamos los nuevos valores de entrada */
	
	/* Nodo Minerals */
	int nminerals = networkBuildingBarracks.FindNode("minerals_building");	
	theProbs.SetSize(2);
	theProbs[0] = few;
	theProbs[1] = (1 - few);	
	networkBuildingBarracks.GetNode(nminerals)->Definition()->SetDefinition(theProbs);	
	
	/* Nodo Is there Barracks? */	
	int nbarracks = networkBuildingBarracks.FindNode("barracks");	
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = yesBarracks;
	theProbs[1] = (1 - yesBarracks);
	networkBuildingBarracks.GetNode(nbarracks)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Building now? */
	int nbuilding = networkBuildingBarracks.FindNode("building_now");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = yesBuildingNow;
	theProbs[1] = (1 - yesBuildingNow);
	networkBuildingBarracks.GetNode(nbuilding)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Time */
	int ntime = networkBuildingBarracks.FindNode("time");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = finished;
	theProbs[1] = (1 - finished);
	networkBuildingBarracks.GetNode(ntime)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Hace more bases? */
	int nbases = networkBuildingBarracks.FindNode("bases");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = yesMoreBases;
	theProbs[1] = (1 - yesMoreBases);
	networkBuildingBarracks.GetNode(nbases)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Distance base to enemy base */
	int ndistance = networkBuildingBarracks.FindNode("distance_base_enemyBase");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = small;
	theProbs[1] = (1 - small);
	networkBuildingBarracks.GetNode(ndistance)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Is there control center? */
	int ncontrolCenter = networkBuildingBarracks.FindNode("controlCenter");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = yesControlCenter;
	theProbs[1] = (1 - yesControlCenter);
	networkBuildingBarracks.GetNode(ncontrolCenter)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Worker */
	int nworker = networkBuildingBarracks.FindNode("worker");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = yesWorker;
	theProbs[1] = (1 - yesWorker);
	networkBuildingBarracks.GetNode(nworker)->Definition()->SetDefinition(theProbs);
	
	/* Actualizamos la red */
	networkBuildingBarracks.UpdateBeliefs();
		
	
	/* Nodo Building barracks */	
	int nbuildingBarracks = networkBuildingBarracks.FindNode("building_barracks");
	
	networkBuildingBarracks.GetNode(nbuildingBarracks)->Value()->GetValue(&value);
	names = networkBuildingBarracks.GetNode(nbuildingBarracks)->Definition()->GetOutcomesNames();
		
	return (*value)[0];
}

/*
 * Ejecuta la red bayesiana de construccion de Factory con los parametros como valores de entrada.
 * @param few Cantidad de minerales relativa a la construccion de edificios.
 * @param yesFactory Indica si la base actual dispone del edifico factory.
 * @param yesBuildingNow Indica si se esta construyendo el edificio factory (en la base actual).
 * @param finished Tiempo transcurrido de la partida.
 * @param yesBarracks Indica si al menos se cuenta con un edificio Barracks.
 * @param yesControlCenter Indica si al menos se cuenta con un edificio control center.
 * @param yesWorker Indica si se dispone de al menos una unidad worker.
 * @return Resultado obtenido al ejecutar la red bayesiana.
 */ 
float Nets::getValueBuildingFactory(float few, float yesFactory, float yesBuildingNow, float finished,
												  float yesBarracks, float yesControlCenter, float yesWorker){
	
	DSL_doubleArray theProbs;
	DSL_idArray * names;
	DSL_Dmatrix * value;
	
	/* Asignamos los nuevos valores de entrada */
	
	/* Nodo Minerals */
	int nminerals = networkBuildingFactory.FindNode("minerals_building");	
	theProbs.SetSize(2);
	theProbs[0] = few;
	theProbs[1] = (1 - few);	
	networkBuildingFactory.GetNode(nminerals)->Definition()->SetDefinition(theProbs);	
	
	/* Nodo Is there factory? */	
	int nfactory = networkBuildingFactory.FindNode("factory");	
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = yesFactory;
	theProbs[1] = (1 - yesFactory);
	networkBuildingFactory.GetNode(nfactory)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Building now? */
	int nbuilding = networkBuildingFactory.FindNode("building_now");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = yesBuildingNow;
	theProbs[1] = (1 - yesBuildingNow);
	networkBuildingFactory.GetNode(nbuilding)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Time */
	int ntime = networkBuildingFactory.FindNode("time");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = finished;
	theProbs[1] = (1 - finished);
	networkBuildingFactory.GetNode(ntime)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Is there barracks? */
	int nbarracks = networkBuildingFactory.FindNode("barracks");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = yesBarracks;
	theProbs[1] = (1 - yesBarracks);
	networkBuildingFactory.GetNode(nbarracks)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Is there control center? */
	int ncontrolCenter = networkBuildingFactory.FindNode("controlCenter");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = yesControlCenter;
	theProbs[1] = (1 - yesControlCenter);
	networkBuildingFactory.GetNode(ncontrolCenter)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Worker */
	int nworker = networkBuildingFactory.FindNode("worker");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = yesWorker;
	theProbs[1] = (1 - yesWorker);
	networkBuildingFactory.GetNode(nworker)->Definition()->SetDefinition(theProbs);
	
	/* Actualizamos la red */
	networkBuildingFactory.UpdateBeliefs();
		
	/* Nodo Building factory */	
	int nbuildingFactory = networkBuildingFactory.FindNode("building_factory");
	
	networkBuildingFactory.GetNode(nbuildingFactory)->Value()->GetValue(&value);
	names = networkBuildingFactory.GetNode(nbuildingFactory)->Definition()->GetOutcomesNames();
	
	return (*value)[0];
}

/*
 * Ejecuta la red bayesiana de entrenamiento de unidades worker con los parametros como valores de entrada.
 * @param few Cantidad de minerales relativa al entrenamiento de unidades.
 * @param finished Tiempo transcurrido de la partida.
 * @param yesControlCenter Indica si al menos se cuenta con un edificio control center.
 * @param yesCanGather Indica se hay vacantes para realizar la tarea gather en la base.
 * @param full Indica si la cola de entrenamiento esta llena.
 * @return Resultado obtenido al ejecutar la red bayesiana.
 */
float Nets::getValueTrainingWorker(float few, float finished, float yesControlCenter, float yesCanGather, float full){
	
	DSL_doubleArray theProbs;	
	DSL_idArray * names;
	DSL_Dmatrix * value;
	
	/* Asignamos los nuevos valores de entrada */
	
	/* Nodo Minerals */
	int nminerals = networkTrainingWorker.FindNode("minerals_training");	
	theProbs.SetSize(2);
	theProbs[0] = few;
	theProbs[1] = (1 - few);	
	networkTrainingWorker.GetNode(nminerals)->Definition()->SetDefinition(theProbs);	
	
	/* Nodo Time */
	int ntime = networkTrainingWorker.FindNode("time");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = finished;
	theProbs[1] = (1 - finished);
	networkTrainingWorker.GetNode(ntime)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Is there control center? */
	int ncontrolCenter = networkTrainingWorker.FindNode("controlCenter");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = yesControlCenter;
	theProbs[1] = (1 - yesControlCenter);
	networkTrainingWorker.GetNode(ncontrolCenter)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Can gather? */
	int ngather = networkTrainingWorker.FindNode("gather");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = yesCanGather;
	theProbs[1] = (1 - yesCanGather);
	networkTrainingWorker.GetNode(ngather)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Queue */
	int nqueue = networkTrainingWorker.FindNode("queue");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = full;
	theProbs[1] = (1 - full);
	networkTrainingWorker.GetNode(nqueue)->Definition()->SetDefinition(theProbs);
	
	/* Actualizamos la red */
	networkTrainingWorker.UpdateBeliefs();
		
	/* Nodo Training worker */	
	int ntrainingWorker = networkTrainingWorker.FindNode("training_worker");
	
	networkTrainingWorker.GetNode(ntrainingWorker)->Value()->GetValue(&value);
	names = networkTrainingWorker.GetNode(ntrainingWorker)->Definition()->GetOutcomesNames();
	
	
	return (*value)[0];
}


/*
 * Ejecuta la red bayesiana de entrenamiento de unidades militares con los parametros como valores de entrada.
 * @param min Representa la cantidad de unidades del mismo tipo.
 * @param ko Indica el ratio actual entre marines y tanks.
 * @param few Cantidad de minerales relativa al entrenamiento de unidades.
 * @param finished Tiempo transcurrido de la partida.
 * @param yesBuilding Indica si se cuenta con el edificio necesario para el entrenamiendo de la unidad.
 * @param noFull Indica si hay hueco en las colas de entrenamiento.
 * @return Resultado obtenido al ejecutar la red bayesiana.
 */
float Nets::getValueTrainingUnit(float min, float ko, float few, float finished, float yesBuilding, float noFull){
	
	DSL_doubleArray theProbs;	
	DSL_idArray * names;
	DSL_Dmatrix * value;
	
	/* Asignamos los nuevos valores de entrada */
	
	/* Nodo Number of units */
	int nunits = networkTrainingUnit.FindNode("units");	
	theProbs.SetSize(2);
	theProbs[0] = min;
	theProbs[1] = (1 - min);	
	networkTrainingUnit.GetNode(nunits)->Definition()->SetDefinition(theProbs);	
	
	/* Nodo Rate */	
	int nrate = networkTrainingUnit.FindNode("rate");	
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = ko;
	theProbs[1] = (1 - ko);
	networkTrainingUnit.GetNode(nrate)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Minerals */
	int nminerals = networkTrainingUnit.FindNode("minerals_training");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = few;
	theProbs[1] = (1 - few);
	networkTrainingUnit.GetNode(nminerals)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Time */
	int ntime = networkTrainingUnit.FindNode("time");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = finished;
	theProbs[1] = (1 - finished);
	networkTrainingUnit.GetNode(ntime)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Is there building? */
	int nbuilding = networkTrainingUnit.FindNode("building");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = yesBuilding;
	theProbs[1] = (1 - yesBuilding);
	networkTrainingUnit.GetNode(nbuilding)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Queue */
	int nqueue = networkTrainingUnit.FindNode("queue");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = noFull;
	theProbs[1] = (1 - noFull);
	networkTrainingUnit.GetNode(nqueue)->Definition()->SetDefinition(theProbs);
	
	
	/* Actualizamos la red */
	networkTrainingUnit.UpdateBeliefs();
		
	
	/* Nodo Training unit */	
	int ntrainingUnit = networkTrainingUnit.FindNode("training_unit");
	
	networkTrainingUnit.GetNode(ntrainingUnit)->Value()->GetValue(&value);
	names = networkTrainingUnit.GetNode(ntrainingUnit)->Definition()->GetOutcomesNames();
	
	
	return (*value)[0];
}


/*
 * Ejecuta la red bayesiana de creacion de una nueva base con los parametros como valores de entrada.
 * @param few Cantidad de minerales relativa a la construccion de edificios.
 * @param force Poder militar.
 * @param small Distancia de la base antigua al cluster de minerales de la nueva. 
 * @param yesMoreBases Indica si se dispone de mas de una base. 
 * @param yesInProgress Indica si se esta llevando a cabo la construnccion de una nueva base.
 * @param finished Tiempo transcurrido de la partida.
 * @param yesPossible Indica si es posible en funcion de la distancia a la base enemiga mas cercana.
 * @param yesWorker Indica si se dispone de al menos una unidad worker.
 * @return Resultado obtenido al ejecutar la red bayesiana.
 */
float Nets::getValueNewBase(float few, float force, float small, float yesMoreBases, float yesInProgress,
										  float finished, float yesPossible, float yesWorker){
	
	DSL_doubleArray theProbs;	
	DSL_idArray * names;
	DSL_Dmatrix * value;
	
	/* Asignamos los nuevos valores de entrada */
	
	/* Nodo Minerals */
	int nminerals = networkNewBase.FindNode("minerals_building");	
	theProbs.SetSize(2);
	theProbs[0] = few;
	theProbs[1] = (1 - few);	
	networkNewBase.GetNode(nminerals)->Definition()->SetDefinition(theProbs);	
	
	/* Nodo Force */	
	int nforce = networkNewBase.FindNode("force");	
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = force;
	theProbs[1] = (1 - force);
	networkNewBase.GetNode(nforce)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Distance old base to cluster minerals */
	int ndistance = networkNewBase.FindNode("distance_oldBase_clusterMineral");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = small;
	theProbs[1] = (1 - small);
	networkNewBase.GetNode(ndistance)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Have more bases? */
	int nbases = networkNewBase.FindNode("bases");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = yesMoreBases;
	theProbs[1] = (1 - yesMoreBases);
	networkNewBase.GetNode(nbases)->Definition()->SetDefinition(theProbs);
	
	/* Nodo In progress? */
	int nprogress = networkNewBase.FindNode("progress");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = yesInProgress;
	theProbs[1] = (1 - yesInProgress);
	networkNewBase.GetNode(nprogress)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Time */
	int ntime = networkNewBase.FindNode("time");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = finished;
	theProbs[1] = (1 - finished);
	networkNewBase.GetNode(ntime)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Is possible? */
	int npossible = networkNewBase.FindNode("possible");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = yesPossible;
	theProbs[1] = (1 - yesPossible);
	networkNewBase.GetNode(npossible)->Definition()->SetDefinition(theProbs);
	
	/* Nodo Worker */
	int nworker = networkNewBase.FindNode("worker");
	theProbs.Flush();
	theProbs.SetSize(2);
	theProbs[0] = yesWorker;
	theProbs[1] = (1 - yesWorker);
	networkNewBase.GetNode(nworker)->Definition()->SetDefinition(theProbs);
	
	/* Actualizamos la red */
	networkNewBase.UpdateBeliefs();
	
	
	/* Nodo New base */	
	int nnewBase = networkNewBase.FindNode("new_base");
	
	networkNewBase.GetNode(nnewBase)->Value()->GetValue(&value);
	names = networkNewBase.GetNode(nnewBase)->Definition()->GetOutcomesNames();
		
	return (*value)[0];
}


/* 
 * Calcula el poder militar.
 * @param nMarines Numero de marines.
 * @param nTanks Numero de tanques.
 * @return Valor del poder militar normalizado, donde 1 significa poder militar maximo
 *  y 0 indica que no se dispone de ninguna unidad militar.
 */
float Nets::getForce(size_t nMarines, size_t nTanks){
	
	float force = nMarines + (nTanks * 3);
	
	force = force / 60;
	
	if(force > 1)
		force = 1.0;
			
	return force;
}


/*
 * Obtiene el tiempo transcurrido de la partida.
 * @param frame Frame actual de la partida.
 * @return Valor del tiempo trasncurrido normalizado, donde 1 significa que la partida
 *  ha finalizado y 0 indica que la partida acaba de comenzar.
 */
float Nets::getTime(int32_t frame){
	assert(frame >= 0);
	
	/* Obtiene el tiempo transcurrido en segundos */
	int32_t seconds = (frame + 1) / 8; /* 8 frames per second */
	float finished = (float)seconds / 1200;
	
	return finished;
}


/*
 * Calcula el tiempo trnscurrido desde el ultimo ataque.
 * @param frameLastAttack Frame del ultimo ataque.
 * @param frame Frame actual de la partida.
 * @return Devuelve el tiempo del ultimo ataque normalizado, donde 1 significa que
 * ha transcurrido al menos 90 segundos y 0 indica que el ultimo ataque enmigo
 * acaba de finalizar.
 */
float Nets::getLastAttack(int32_t frameLastAttack, int32_t frame){
	assert(frame >= 0);
	assert(frameLastAttack >= 0);
	
	/* Obtiene el tiempo transcurrido en segundos */
	float longTime = (float)(frame - frameLastAttack)  / 8;
	longTime = longTime / 90;
	
	if(longTime > 1.0)
		longTime = 1.0;
	
	return longTime;
}

/*
 * Obtiene la cantidad de minerales disponible para la construccion de edificios.
 * @param minerals Cantidad de minerales disponibles.
 * @return Valor normalizado de la cantidad de minerales disponibles, donde 1 
 * significa que no se dispone de ningun mineral y 0 indica que se dipone
 * de al menos 2800 minerales.
 */
float Nets::getMineralsBuildingFew(uint32_t minerals){

	float many = (float) minerals / 2800;
	
	if(many > 1.0)
		many = 1.0;
		
	float few = 1 - many;

	return few;
}


/*
 * Calcula la distancia de la posicion indicada al edificio enemigo mas cercano.
 * @param centreLoc Posicion sobre la cual se va a buscar el edificio enemigo mas cercano.
 * @param locationsEnemyBuilding Posiciones de los edificios enemigos.
 * @return Distancia normalizada al edificio enemigo mas cercano, donde 1 
 * significa que la distancia es de cero puntos y 0 indica que la distancia es
 * de 1448 puntos.
 */
float Nets::getSmallDistanceToEnemyBase(TerrainBase::Loc centreLoc, Vector<ScalarPoint> locationsEnemyBuilding){

	float small;
	float big;
	float min = -1;	
	ScalarPoint point;
	
	FORALL(locationsEnemyBuilding, location){
				
		float distance = sqrt(square(centreLoc.x - location->x) + square(centreLoc.y - location->y));
		
		if((min == -1) || (distance < min)){
			min = distance;
			point = *location;
		}	
	}
	
	if(min == -1){
		min = 1448;	
	}
		
	big = (float) min / 1448;
	small = 1 - big;
	
	return small;
}


/*
 * Obtiene la cantidad de minerales disponible para el entrenamiento de unidades.
 * @param minerals Cantidad de minerales disponibles.
 * @return Valor normalizado de la cantidad de minerales disponibles, donde 1 
 * significa que no se dispone de ningun mineral y 0 indica que se dipone
 * de al menos 600 minerales.
 */
float Nets::getMineralsTrainingFew(uint32_t minerals){
		
	float many = (float) minerals / 600;
	
	if(many > 1.0)
		many = 1.0;
	
	float few = 1 - many;
		
	return few;	
}


/*
 * Obtiene el numero normalizado de unidades de un determinado tipo.
 * @param bp_name Nombre del tipo de unidad.
 * @param count Numero de unidades del mismo tipo.
 * @return Nmero de unidades normalizado, donde 1 significa que no se dipone de ninguna unidad
 * y 0 indica que al menos se dispone de 40 (13) marines (tanques).
 */
float Nets::getMinNumberOfUnit(const std::string & bp_name, size_t count){

	float min = 0;
	float max = 0;
	
	if(bp_name == "marine"){
		max = (float) count / 40;
		
	}else if(bp_name == "tank"){
		max = (float) count / 13;		
	}
	
	if(max > 1.0)
		max = 1.0;
	
	min = 1.0 - max;
		
	return min;
}

/*
 * Obtiene el ratio entre las unidades marine y tank.
 * @param bp_name Indica sobre que unidad se desea calcular el ratio.
 * @param countMarines Numero de marines.
 * @param countTanks Numero de tanques.
 * @return Valor del ratio normalizado. Vease memoria del PFC, apartado 4.3.6
 */
float Nets::getKoRatio(const std::string & bp_name, size_t countMarines, size_t countTanks){
	
	float ko = 0;
	float ok = 0;
		
	if(bp_name == "marine"){
				
		if((countMarines == 0) && (countTanks == 0))
			ok = 0.0;
		else if((countMarines != 0) && (countTanks == 0))
			ok = 3.0;
		else if(countTanks != 0)
			ok = (float) countMarines / countTanks;
		
		if(ok > 3)
			ok = 3.0;
			
		ok = ok / 3;
		
		ko = 1.0 - ok;
		
	}else if(bp_name == "tank"){
				
		if(countTanks == 0)
			ok = 6.0;
		else
			ok = (float) countMarines / countTanks;
			
		if(ok < 3.0)
			ok = 3.0;
			
		if(ok > 6.0)
			ok = 6.0;
			
		ok = ok / 3;
		
		ko = ok - 1;
		
		ok = 1 - ko;
		
	}	
	
	return ko;	
}
