/* Codigo base: Librerias GatherCommander */
/* Sergio Nuñez Covarrubias */


#ifndef SimpleGather_H
#define SimpleGather_H

#include "Random.H"
#include "Game.H"
#include "Movement.h"
#include "ModuleSet.h"
#include "Movement.H"
#include "MineralCluster.H"
#include "Gather_accessories.H"
#include "Delegate.H"
#include "SpecialWorker.H"
#include "SpecialRouteAllocator.H"
#include "History.H"
#include "Watcher.H"
#include "WatcherAux.H"

class SpecialWorker;
class GameStateModule;
class GfxModule;


typedef boost::shared_ptr<SpecialRouteAllocator> SpecialRouteAllocatorPtr;
typedef boost::shared_ptr<MineralCluster> MineralClusterPtr;


/* Definicion de la clase mineral */
class Mineral {
public:
    Vector<TerrainBase::Loc> points;
    TerrainBase::Loc centre;
    Mineral(Vector<TerrainBase::Loc> points_, TerrainBase::Loc centre_)
	: points(points_), centre(centre_)
    {}
    ~Mineral(){}
};


typedef boost::shared_ptr<Mineral> MineralPtr;


/* Estructura de soporte para la tarea gather */
struct SpecialGather{
	
	/* workers */
	Vector<SpecialWorker*> workers;
	
	/* ControlCenter where deposit the minerals */
	GameObj* controlCenter;
	
	/* Cluster of Mineral where the workers must mining */	
	boost::shared_ptr<MineralCluster> cluster;
	
	/* Constructor */
	SpecialGather(Vector<SpecialWorker*> workers_, GameObj* controlCenter_, 
				  boost::shared_ptr<MineralCluster> cluster_): workers(workers_){
		controlCenter = controlCenter_;
		cluster = cluster_;		
	}	
};


/* Deinicion de la clase SimpleGather */
class SimpleGather: public EventHandler{

public:

	/* Vector global de control centers */
	Vector<GameObj*>* controlCenters;
		 
	/* Propotipo del metodo handle_event */
	bool handle_event(const Event &e);
	
	/* Propotipo del metodo moveTo */
	void moveTo(GameObj *unit, GameObj *gob);	
	
	/* Propotipo del metodo moveTo */
	void moveTo(GameObj *unit, const TerrainBase::Loc &location);
		
	/* Propotipo del metodo gather */
	void gather();	
	
	/* Propotipo del metodo addWorkersGather */
	void addWorkersGather(GameObj* gob, GameObj* controlCenter, boost::shared_ptr<MineralCluster> cluster);
	
	/* Propotipo del metodo removeWorkerGather */
	GameObj* removeWorkerGather();
		
	/* Propotipo del metodo getNumberWorkersGather */
	unsigned getNumberWorkersGather();
	
	/* Propotipo del metodo getNumberWorkers */
	unsigned getNumberWorkers(unsigned i);
	
	/* Propotipo del metodo getMineralDied */
	bool getMineralDied();
	
	/* Propotipo del metodo isEmptyNewMinerals */
	bool isEmptyNewMinerals();
	
	/* Propotipo del metodo administerMinerals */
	void administerMinerals();
	
	/* Propotipo del metodo getNumberMineralClusters */
	unsigned getNumberMineralClusters();
	
	/* Propotipo del metodo addControlCenter */
	void addControlCenter(GameObj* cc);
	
	/* Propotipo del metodo init */
	void init();
	
	/* Propotipo del metodo countWorkers */
	size_t countWorkers();
	
	/* Propotipo del metodo getMineralClusters */
	Vector<boost::shared_ptr<MineralCluster> > getMineralClusters();
	
	/* Propotipo del metodo newChanges */
	bool newChanges();	
	
	/* Propotipo del metodo setChanges */
	void setChanges(bool value);
	
	/* Propotipo del metodo canGather */
	float canGather(Vector<boost::shared_ptr<MineralCluster> > clusters, size_t workersQueued);
	
	/* Propotipo del metodo getBestCluster */
	boost::shared_ptr<MineralCluster> getBestCluster(Vector<boost::shared_ptr<MineralCluster> > clusters);
	
	/* Propotipo del metodo comprobarWorkersGather */
	void comprobarWorkersGather();
	
	/* Propotipo del metodo computeRoutes */
	void computeRoutes(int indexControlCenters);
	
	/* Propotipo del metodo onLoseUnit */
	void onLoseUnit(GameObj * unit);
	
	/* Constructor */
	SimpleGather(GameStateModule * gsm_, sint4 seed, Vector<GameObj*>* vector, ModuleSet & msGather, Vector<GameObj*>* vectorCC): 
	 moduleSet(msGather), gsm(gsm_), mcGather(msGather.get_movement(), "Default", "Default"), history(msGather.get_history()){
		rand.srand(seed);		
		mineralDied = false;
		routeAllocator = SpecialRouteAllocatorPtr(new SpecialRouteAllocator());
		workers = vector;
		controlCenters = vectorCC;
		msGather.get_history().trackBlueprint("mineral");
		changes = false;
	}
	
	/* Destructor */
	~SimpleGather(){
		moduleSet.get_watcher().cancelNotify(newMineralsWatchID);
	}

private:

	/* Identificador del "watcher" sobre la creacion de minerales */
	uint4 newMineralsWatchID;
	
	/* Conjunto de modulos de apoyo */
	ModuleSet& moduleSet;
	
	/* Modulo GameStateModule */
	GameStateModule * gsm;
	
	/* Modulo de apoyo para el movimiento */
	Movement::Context mcGather;
	
	/* Vector de los grupos para el gather */
	Vector<SpecialGather> workersGather;
	
	/* Modulo de apoyo History */
	History::HistoryModule &history;

	/* Vector global de unidades workers */	
	Vector<GameObj*>* workers;

	// Contains any minerals that came into view (or out of fog of war) since the last frame.
	std::set<GameObj*> newMinerals;
	
	bool mineralDied;
	
	// a vector of mineral clusters containing organised Minerals
	Vector<boost::shared_ptr<MineralCluster> > mineralClusters;
	
	// Contains the History records.  All calls to minerals are done through these
	// records (for safety in fog of war).
	std::set<const History::Record*> mineralRecs;
	
	// a class that sorts and allocates the routes
	boost::shared_ptr<SpecialRouteAllocator> routeAllocator;
	
	Random rand;
	
	bool changes;
	
	/* Prototipo del metodo computeMineralClusters */
	void computeMineralClusters();
	
	
	/**
     * called whenever a status update is received
     * @param task the task object whose status has changed
     */
    void handleMovementStatus(const Movement::Task & task);
    
    /**
     * called whenever a unit cannot move to its next point
     * @param worker the worker that cannot move
     */
    void handleMoveFailure(SpecialWorker *worker);
	
    /**
     * called whenever a unit cannot find a path to its destination
     * @param worker the worker that cannot find a path
     */
    void handlePlanFailure(SpecialWorker *worker);
	
    /**
     * called whenever a unit reaches its destination.
     * @param worker the worker that reached its destination
     */
    void handleSuccess(SpecialWorker *worker);
	
	/* Prototipo del metodo getDistance */
	real8 getDistance(GameObj *obj, TerrainBase::Loc loc);
	
	/* Prototipo del metodo moveRandomly */
	void moveRandomly(GameObj *unit);
	
	/* Prototipo del metodo onLoseControlCenter */
	void onLoseControlCenter(unsigned index, GameObj* cc);
	
	/* Prototipo del metodo removeWorkerGather */
	void removeWorkerGather(GameObj* gob);	
};

#endif
