/* Codigo base: BaseCommander.C -  Anthony Marcar */
/* Sergio Nuñez Covarrubias */

#include <string>
#include "SpecialBaseCommander.H"
#include "GameStateModule.H"
#include "Game.H"
#include "SpecialBuildCommander.H"
#include "SpecialDefenseCommander.H"
#include "TechTree.H"

SpecialBaseCommander::Building::Building(ModuleSet & ms, SpecialBaseCommander * base, GameObj * building) :
    ms(ms), base(base), building(building), training(), funds(NULL),  watch(Watcher::NONE), confirmed(false), complete(true){
    int x0, y0, x1, y1;
    building->get_p1(x0, y0);
    building->get_p2(x1, y1);
    x0 -= 32; y0 -= 32;
    x1 += 32; y1 += 32;
    sint4 me = ms.get_game_state().get_game().get_client_player();
    watch = ms.get_watcher().notifyOnCreation(base, Watcher::And(Watcher::OwnerIs(me), Watcher::InRectangle(x0, y0, x1, y1)), 100, false);
}

SpecialBaseCommander::Building::~Building(){
    if(funds != NULL){
        funds->release();
    }

    if(watch != Watcher::NONE){
        ms.get_watcher().cancelNotify(watch);
    }
}

GameObj * SpecialBaseCommander::Building::getBuilding(){
    return building;
}

bool SpecialBaseCommander::Building::isBusy() const{
    return (!complete);
}

const std::string & SpecialBaseCommander::Building::getTrainingBP() const{
    return training;
}

bool SpecialBaseCommander::Building::canTrain(std::string bp_name) const{
    return TechTree::GetTrainer(bp_name) == building->bp_name();
}

void SpecialBaseCommander::Building::train(std::string bp_name, Finance::Ticket * funds){
    assert( !isBusy() );
    assert( canTrain(bp_name) );
    assert( funds->available() >= ms.get_finance().get_cost(bp_name) );

    training = bp_name;
    action = TechTree::GetTrainAction(bp_name);
    this->funds = funds;
    confirmed = false;
    complete = false;

    building->set_action(action, Vector<sint4>());
}

Watcher::WatchID SpecialBaseCommander::Building::getWatch() const{
    return watch;
}
 
void SpecialBaseCommander::Building::update(){
    const stype * active_ptr = building->get_val_ptr("active");
    assert(active_ptr != NULL);
    bool active = ( active_ptr->to_int() == 1 );

    if(!confirmed){
        if(active){
            confirmed = true;
            funds->spend();
            funds = NULL;
        
		}else{
            if(action != "")
				building->set_action(action, Vector<sint4>());
        }
    
	}else if(!complete){
        if(!active){
            complete = true;            
        }
    }
}

SpecialBaseCommander::SpecialBaseCommander(ModuleSet& ms, Vector<GameObj*> units, TerrainBase::Loc centre, 
										   Vector<GameObj*>* vectorCC, Vector<GameObj*>* vectorWorkers,
										   Vector<GameObj*>* vectorMarines, Vector<GameObj*>* vectorTanks, sint4 updateTime_)    
	: SpecialCommander(ms, updateTime_), centreLoc(centre), radius(150), askingForBuilder(false){	
	
    unitsQueued["worker"] = 0;
    unitsQueued["marine"] = 0;
    unitsQueued["tank"] = 0;
    unitsQueued["controlCenter"] = 0;
    unitsQueued["barracks"] = 0;
    unitsQueued["factory"] = 0;
	controlCenters = vectorCC;
	workers = vectorWorkers;
	marines = vectorMarines;
	tanks = vectorTanks;
	
	defender = NULL;

    init(units);
}

SpecialBaseCommander::~SpecialBaseCommander(){
}

/////////////////////////////////private////////////////////////////////
void SpecialBaseCommander::onReceiveUnit( GameObj * unit, Watcher::WatchID watchID){
    FORALL( obj2building, o2b){
        Building * building = o2b->second;
        if( building->getWatch() == watchID ){
            unitsQueued[unit->bp_name()]--;
        }
    }

    if (unit->bp_name() == "worker"){
        if (askingForBuilder){
			
			if (unit && unit->sod.in_game && *unit->sod.x >= 0 && !unit->is_dead()) {
				moduleSet.get_watcher().notifyOnDeath(this, unit);
				builders.push_back(unit);				
				addUnit2(unit);
				askingForBuilder = false;				
			}
		}else{
			workers->push_back(unit);			
		}
						
    
	}else if (unit->bp_name() == "controlCenter" || unit->bp_name() == "barracks" || unit->bp_name() == "factory"){
        Building * building = new Building(moduleSet, this, unit);
        obj2building.insert(std::make_pair(unit, building));
				
		TerrainBase::Loc centerGob;
		unit->get_center(centerGob.x, centerGob.y);
		
		setNewCentre(centerGob);
		setNewRadius();
				
		Vector<GameObj*> vectorGob;
		
		sint4 lastAttack = 0;
		bool underAttack = false;
		
		if(defender != NULL){
			underAttack = defender->isUnderAttack();
			lastAttack = defender->getLastAttack();
			vectorGob = defender->getAllUnits();
			delete defender;
			defender = NULL;
		}
				
		defender = new SpecialDefenseCommander(this, centreLoc, radius, lastAttack, underAttack);
		defender->addUnits(vectorGob);
		
		if (unit->bp_name() == "controlCenter")
			controlCenters->push_back(unit);		
    
	}else if (unit->bp_name() == "marine"){
        marines->push_back(unit);
				
	}else if (unit->bp_name() == "tank"){
        tanks->push_back(unit);
    
	}else{ // Should never happen    
        assert( false ); 
    }
}

void SpecialBaseCommander::onLoseUnit( GameObj * unit){
    std::map<GameObj*, Building*>::iterator it;
    
	if( (it = obj2building.find(unit)) != obj2building.end()){
        // If we had a unit training at this building
        // Be sure to record that it is no longer training
        Building * building = it->second;
        building->update();
        if(building->isBusy()){
            unitsQueued[building->getTrainingBP()]--;
        }
        delete building;

        obj2building.erase(unit);
    }

	Vector<GameObj*>::iterator it2;
	
	if((it2 = builders.find(unit)) != builders.end()){
		builders.erase(it2);		
	}	
	
	if((unit->bp_name() == "controlCenter") && ((it2 = controlCenters->find(unit)) != controlCenters->end())){
		controlCenters->erase(it2);		
	}   
}

void SpecialBaseCommander::buildBuildingsInQueue(){   
    	
	while ( !buildingsToBuild.empty()){
	
        // What are we going to build?
        BuildTask task = buildingsToBuild.front();		
  
		if(builders.size() > 0){
           
            std::cerr << "Found one!" << std::endl;
			
            // Transfer him over to a new build commander
            
			GameObj * worker = builders.back();
			builders.pop_back();				
            removeUnit(worker);
			
			TerrainBase::Loc workerLoc;
			worker->get_center(workerLoc.x, workerLoc.y);
			
            SpecialBuildCommander * buildCommander = new SpecialBuildCommander(this, worker, task.bp_name, workerLoc, task.funds, 5);
            activeBuilders.insert(buildCommander);

            // And we are done for this building
            buildingsToBuild.pop();
        
		}else{           
            return; // Can't build right now, return early
        }
    }
}

void SpecialBaseCommander::buildUnitsInQueue(){

    // TODO: Optimize so we don't wait on a free control center
    // simply because we have a marine queued and the barracks is busy
    while( !unitsToBuild.empty() ){
	
        BuildTask task = unitsToBuild.front();

        Building * building = NULL;
        FORALL(obj2building, b){
            Building * current = b->second;
            if(current->isBusy() == false && current->canTrain(task.bp_name)){
                building = current;
                break;
            }
        }

        if( building != NULL ){ // If we have a building
        
			building->train(task.bp_name, task.funds);
            unitsToBuild.pop();
        }else{ // Otherwise
			return; // Cannot train any more just yet
        }      
    }   
}

void SpecialBaseCommander::exec(){

    FORALL(finishedBuilders, b){
        SpecialBuildCommander * builder = (*b);
        unitsQueued[ builder->getBuildingName() ]--;
        delete builder;
    }
	
    finishedBuilders.clear();

    FORALL(obj2building, o2b){
        o2b->second->update();
    }

    if (!unitsToBuild.empty()){
        buildUnitsInQueue();
    }

    if (!buildingsToBuild.empty()){
        buildBuildingsInQueue();
    }
}

void SpecialBaseCommander::init(const Vector<GameObj*> & initialUnits){

    std::set<GameObj*> unitsToAssign(initialUnits.begin(), initialUnits.end());
    // Distribute any remaining units as we see fit
    addUnits( unitsToAssign );//Finalmente llama a onReceiveUnit( unit, Watcher::NONE );
	//defender = new SpecialDefenseCommander(this, centreLoc, radius);
}

//////////////////////////public////////////////////////////

void SpecialBaseCommander::buildUnit(const std::string & unitBPname, Finance::Ticket * funds){
    // Reserve enough funds to build the building
    Finance::Module & fm = moduleSet.get_finance();
    uint4 unitCost = fm.get_cost(unitBPname);
    assert( funds->available() >= unitCost );
    funds->trim(unitCost);

    unitsToBuild.push( BuildTask(unitBPname, funds) );
    unitsQueued[unitBPname]++;
}

void SpecialBaseCommander::buildBuilding(const std::string & buildingBPname, Finance::Ticket * funds){
    // Reserve enough funds to build the building
    Finance::Module & fm = moduleSet.get_finance();
    uint4 buildingCost = fm.get_cost(buildingBPname);
    assert( funds->available() >= buildingCost );
    funds->trim(buildingCost);

    buildingsToBuild.push( BuildTask(buildingBPname, funds) ); 
    unitsQueued[buildingBPname]++;
}

bool SpecialBaseCommander::canBuild(const std::string & objectBPname){

    // If a building, determine its builder
    const std::string & breq = TechTree::GetBuilder(objectBPname);
    if( breq != "" ){		
        return (this->countOwned(breq) > 0);
    }

    const std::string & treq = TechTree::GetTrainer(objectBPname);
    if( treq != ""){		
        return (this->countOwned(treq) > 0);
    }

    return false;
}

/**
 * Asigna un nuevo centro a la base. Simplemente calcula el punto
 * intermedio entre el centro anterior de la base y el punto
 * que se recibe por parametros (normalmente el centro de un nuevo edifico).
 * @param centerGob Punto sobre el cual se va a calcular el centro de la base.
 */
void SpecialBaseCommander::setNewCentre(TerrainBase::Loc centerGob){	
	
	TerrainBase::Loc centerAux(centreLoc);	
			
	/* Comprobamos que su centro no sea el centro actual */
	if((centerGob.x != centreLoc.x) || ((centerGob.y != centreLoc.y))){		
			
		/* Calculamos el centro entre el anterior y el edificio actual */
		float dx = sqrt(square(centerGob.x - centerAux.x));
		float dy = sqrt(square(centerGob.y - centerAux.y));				
				
		dx /= 2;
		dy /= 2;				
				
		/* Coordenada X */
		if(centerGob.x < centerAux.x)
			centerAux.x = centerGob.x + dx;
		else
			centerAux.x = centerAux.x + dx;
					
		/* Coordenada Y */
		if(centerGob.y < centerAux.y)
			centerAux.y = centerGob.y + dy;
		else
			centerAux.y = centerAux.y + dy;		
			
	}		
	
	centreLoc.x = centerAux.x;
	centreLoc.y = centerAux.y;	
}

/** 
 * Asigna un nuevo radio a la base. Dicho radio es la
 * distancia del centro de la base al edificio mas lejano.
 */
void SpecialBaseCommander::setNewRadius(){
	
	/* Si solo hay un elemento el radio es el inicial */
	if(obj2building.size() != 1){
		
		sint4 aux = 0;
		GameObj* gob;
		TerrainBase::Loc centerGob;
		
		/* Para todos los edificios de la base */
		FORALL(obj2building, it){
			gob = it->first;
			
			/* Obtenemos el centro del edificio */
			gob->get_center(centerGob.x, centerGob.y);
			
			/* Calculamos la distancia del edificio al centro de la base */
			sint4 distance = sqrt(square(centerGob.x - centreLoc.x) + square(centerGob.y - centreLoc.y));
			
			if(distance > aux)
				aux = distance;		
		}
		
		radius = aux + 150;	
	}
}

/**
 * Receptor de eventos.
 * @param e Evento ocurrido.
 */
bool SpecialBaseCommander::onEvent(const Event &event){

    if (event.get_who() == SpecialBuildCommander::FROM){
		/* Construccion terminada */
        if (event.get_what() == SpecialBuildCommander::FINISHED){
            
            const BuildFinishedEvent & e = 
                dynamic_cast<const BuildFinishedEvent &>(event);
            SpecialBuildCommander * commander = e.getSource();
            
			/* Se libera la unidad worker */
			GameObj* gob = e.getWorker();
			if (gob && gob->sod.in_game && *gob->sod.x >= 0 && !gob->is_dead()) {
				workers->push_back(gob);
			}
						
            activeBuilders.erase(commander);
            finishedBuilders.insert(commander);
        
		/* Construccion fallida */
		}else if (event.get_what() == SpecialBuildCommander::FAILED){
            
            const BuildFailedEvent & e = 
                dynamic_cast<const BuildFailedEvent &>(event);
            SpecialBuildCommander * commander = e.getSource();
			
			/* Se libera la unidad worker */
			GameObj* gob = e.getWorker();
			if (gob && gob->sod.in_game && *gob->sod.x >= 0 && !gob->is_dead()) {
				workers->push_back(gob);
			}
            
            activeBuilders.erase(commander);
            finishedBuilders.insert(commander);
        }
    }
    return false;   
}


/**
 * Cuenta el numero de unidades indicado que se encuentran en
 * la cola de entrenamiento.
 * @param bp_name Indica el tipo de unidad del que se quiere saber 
 * el numero de unidades.
 * @return numero de unidades disponibles del tipo indicado.
 */
size_t SpecialBaseCommander::countQueued(const std::string & bp_name) const{
 
	std::map<std::string, size_t>::const_iterator it;
    it = unitsQueued.find(bp_name);
    if(it != unitsQueued.end()){
        return it->second;
    
	}else{
        return 0;
    }
}

/**
 * Asigna un nuevo valor a la variable askingForBuilder.
 * @param value Nuevo valor a asignar.
 */
void SpecialBaseCommander::setAskingForBuilder(bool value){
	askingForBuilder = value;
}

/**
 * Indica si la defensa de la base ha sido creada y esta operativa.
 * @return true en caso afirmativo, false en caso contrario.
 */
bool SpecialBaseCommander::isDefenderOperative(){
	if(defender != NULL)
		return true;
	else
		return false;
}


/**
 * Añade una unidad militar a la defensa de la base.
 * @param gob Unidad a añadir.
 */
void SpecialBaseCommander::addUnitToDefender(GameObj* gob){
	defender->addUnit(gob);
}


/**
 * Cuenta el numero de unidades indicado. Si el nombre de la unidad 
 * corresponde con "", se cuentan todas las unidades disponibles, 
 * sin tener en cuenta el tipo.
 * @param bp_name Indica el tipo de unidad del que se quiere saber 
 * el numero de unidades.
 * @return numero de unidades disponibles del tipo indicado.
 */
size_t SpecialBaseCommander::countUnitsDefender(const std::string & bp_name){
	if(defender != NULL)	
		return defender->countUnits(bp_name);
	else
		return 0;
}


/**
 * Indica si la region de defensa esta bajo control, es decir, 
 * si al menos el 75% de las unidades estan dentro de la region 
 * y no hay ningun enemigo.
 * @return true si la region esta bajo control, false en caso contrario.
 */
bool SpecialBaseCommander::isBaseUnderControl(){
	if(defender != NULL)
		return defender->isUnderControl();
	else
		return false;
}


/**
 * Obtiene el numero de edificios que forman la base.
 * @return Numero de edificios activos en la base.
 */
size_t SpecialBaseCommander::countBuilding(){
	return obj2building.size();
}


/**
 * Obtiene el frame del ultimo ataque recibido.
 * @return Frame del ultimo ataque.
 */
sint4 SpecialBaseCommander::getLastAttack(){
	return defender->getLastAttack();
}


/**
 * Indica si en la region de defensa hay alguna unidad enemiga.
 * @return true en caso afirmativo, false en caso contrario.
 */
bool SpecialBaseCommander::isBaseUnderAttack(){	
	return defender->isUnderAttack();	
}

/**
 * Indica si hay algun edificio del tipo indicado.
 * @param Tipo del edificio de la consulta.
 * @return 1.0 si existe al menos uno, 0.0 si no existe ninguno.
 */
float SpecialBaseCommander::isThereBuilding(std::string bp_name){

	float success = 0.0;

	FORALL(obj2building, b){
		GameObj* unit = b->first;
		
		if(unit->bp_name() == bp_name)
			success = 1.0;
	}
	
	return success;
}


/**
 * Indica si se esta construyendo un edificio del tipo indicado.
 * @param Tipo del edificio de la consulta.
 * @return 1.0 Si se estan construyendo y 0.0 en caso contrario.
 */
float SpecialBaseCommander::buildingNow(std::string bp_name){

	float buildingNow = 0.0;

	if(unitsQueued[bp_name] > 0)
		buildingNow = 1.0;

	return buildingNow;
}

/**
 * Obtiene las coordenadas del centro del edificio control center.
 * En caso de no disoner en el instante actual, se utilizara el
 * centro de la base como coordenadas del control center.
 * @return Coordendas del centro del cc.
 */
TerrainBase::Loc SpecialBaseCommander::getLocationControlCenter(){
	TerrainBase::Loc loc;
	loc.x = -1;
	loc.y = -1;
	
	FORALL(obj2building, b){
		GameObj* unit = b->first;
		
		if(unit->bp_name() == "controlCenter")
			unit->get_center(loc.x, loc.y);
	}
	
	/* Si no hay cc, pueden pasar dos cosas. O que la base se este construyendo 
	 y todavia no se haya terminado la construccion del cc o que lo hayan
	 destruido. Por ello, se utilizara como posicion del cc el centro de la base,
	 ya que si lo han destruido, lo mas probable es que la base se elimine y
	 las unidades de la defensa se dirijan a otra base. */
	if((loc.x == -1) && (loc.y == -1)){
		loc.x = centreLoc.x;
		loc.y = centreLoc.y;
	}
	
	return loc;	
}

/**
 * Obtiene el control center de la base.
 * @return Control center de la base o null si no se dispone de ninguno.
 */
GameObj* SpecialBaseCommander::getControlCenter(){
	GameObj* cc = NULL;
	
	FORALL(obj2building, b){
		GameObj* unit = b->first;
		
		if(unit->bp_name() == "controlCenter")
			cc = unit;
	}	
	
	return cc;
}

/**
 * Obtiene el numero de edificios del tipo indicado.
 * @param bp_name Tipo del edificio del que se desea conocer el
 * numero de unidades disponibles.
 * @return Numero de edificios del tipo indicado.
 */
size_t SpecialBaseCommander::countBuildings(const std::string & bp_name){

	size_t count = 0;	
	
	FORALL(obj2building, b){
		GameObj* unit = b->first;
		
		if(unit->bp_name() == bp_name)
			count++;
	}
	
	return count;
}


/**
 * Si la defensa de la base aun no ha sido creada, se crea con las
 * unidades recibidas por parametros.
 * @param units Conjunto de unidades militares para la defensa de la base.
 */
void SpecialBaseCommander::createDefense(Vector<GameObj*> units){

	if(defender == NULL)
		defender = new SpecialDefenseCommander(this, centreLoc, radius, 0, false);
		
	FORALL(units, u)
		defender->addUnit(*u);

}

/**
 * Obtiene a/3 unidades de cada tipo. Dichas unidades ademas son eliminadas de la defensa.
 * @param a Indica el numero de tercios de unidades que se desean obtener.
 * @return Vector con todas las unidades demandadas.
 */
Vector<GameObj*> SpecialBaseCommander::getUnitsToNewBase(float a){
	return defender->getUnitsToNewBase(a);
}


