/* Codigo base: DefenseCommander.C */
/* Sergio Nuñez Covarrubias */

#ifndef __SPECIAL_DEFENSE_COMMANDER_H__
#define __SPECIAL_DEFENSE_COMMANDER_H__

#include "SpecialCommander.H"
#include "Game.H"
#include "Watcher.H"
#include "WatcherAux.H"
#include "ClientModule.H"
#include "TerrainBase.H"
#include "GridAbstraction.H"
#include "Movement.H"

using namespace Watcher;

class SpecialDefenseCommander : public SpecialCommander{

public:
    
	enum UNITSTATE { STOPPED,     // Unit is not moving
                     CHASING,     // Unit is pathing towards a target
                     ATTACKING,   // Unit is under control of shooter
                     RETURNING }; // Unit is pathing towards the zone
private:
    
	/* Mapa que almacena las unidades de la defensa y el estado de cada una de ellas */
	std::map<GameObj*, UNITSTATE> unit2state;

	/* Coordenada de accion de la defensa */	
    TerrainBase::Loc center;
	
	/* Radio de accion */
    sint4 radius;
		
    sint4 me;
	
	/* Indica si la region definida por center y radius esta bajo control
	 (unidades dentro de la regios y ninguna unidad enmiga a la vista) */
    bool underControl;

	/* Modulo de apoyo para el movimiento */
    Movement::Context mc;
    
    TerrainAnalysis::GridAbstraction ga;
	
	/* Frame del ultimo ataque */
	sint4 lastAttack;
	
	/* Indica si en el radio de accion hay algun enemigo */
	bool underAttack;
		
public:	
	
	/* Constructores y Destructor */
    SpecialDefenseCommander(SpecialCommander * mySuperior, const TerrainBase::Loc & center, sint4 radius, sint4 valueLastAttack, bool valueUnderAttack);
	SpecialDefenseCommander(ModuleSet & ms, const TerrainBase::Loc & center, sint4 radius, sint4 valueLastAttack, bool valueUnderAttack);
    ~SpecialDefenseCommander();
		
	/* Prototipo del metodo pickClosest*/
    GameObj * pickClosest(const Watcher::UnitPredPtr & pred, const std::set<GameObj*> & candidates) const;
        
	/* Prototipo del metodo pickBestTarget */
    GameObj * pickBestTarget(const std::set<GameObj*> & candidates) const;

	/* Prototipo del metodo exec */
    void exec(); 
                 
	/* Prototipo del metodo onReceiveUnit */
    void onReceiveUnit(GameObj * unit, Watcher::WatchID);

	/* Prototipo del metodo onLoseUnit */
    void onLoseUnit(GameObj * unit);

	/* Prototipo del metodo onEvent */
    bool onEvent(const Event & event);

	/* Prototipo del metodo onMovement */
    void onMovement(const Movement::Task & task);	
    
    /// Return true if all units are within the zone and no enemies are
    /// detected inside the boundaries of the zone.
    bool isUnderControl() const;
	
	/* Prototipo del metodo getAllUnits */
	Vector<GameObj*> getAllUnits();
	
	/* Prototipo del metodo getUnit */
	GameObj* getUnit();
	
	/* Prototipo del metodo countUnits */
	size_t countUnits(const std::string & bp_name);
	
	/* Prototipo del metodo getLastAttack */
	sint4 getLastAttack();
	
	/* Prototipo del metodo isUnderAttack */
	bool isUnderAttack() const;
	
	/* Prototipo del metodo getUnitsToNewBase */
	Vector<GameObj*> getUnitsToNewBase(float a);
	
};

#endif
