/* Codigo base: Route.C */
/* Sergio Nuñez Covarrubias */

#include "SpecialRoute.H"
#include "SimpleGather.H"

#define TEST(x) x

SpecialRoute::SpecialRoute(SimpleGather *commander_, const History::Record *mineralRec_,
            TerrainBase::Loc accessPoint_, GameObj *cc_, sint4 max):
			 commander(commander_),mineralRec(mineralRec_), accessPoint(accessPoint_),
		 	 controlCenter(cc_), currentWorkers(0), maxWorkers(max), routeLength(0){ 
    setRouteLength(getSimpleRouteLength());
}


SpecialRoute::~SpecialRoute() { }


/**
 * increments the number of workers on this route
 */
void SpecialRoute::addWorker(){
    currentWorkers++;
}


/**
 * returns the access point of this route
 */
TerrainBase::Loc SpecialRoute::getAccessPoint() const{
    return accessPoint;
}


/**
 * returns the control center associated with this route
 */
GameObj* SpecialRoute::getCC() const{
    return controlCenter;
}


/**
 * returns the number of workers assigned to this route
 */
sint4 SpecialRoute::getCurrentWorkers() const{
    return currentWorkers;
}


/**
 * returns the id of this route
 */
sint4 SpecialRoute::getID() const{
    return id;
}


/**
 * returns the maximum number of workers that can be assigned to this route
 */
sint4 SpecialRoute::getMaxWorkers() const{
    return maxWorkers;
}


/**
 * returns the history record of the mineral associated with this route
 */
const History::Record* SpecialRoute::getMineralRec() const{ 
    return mineralRec; 
}


/**
 * Returns the real length of this route based on path finding rather than dist
 */
real8 SpecialRoute::getRealRouteLength() const{
    /*
    TerrainBase::Loc ccLoc;
    controlCenter->get_center(ccLoc.x, ccLoc.y);
    real8 dist = commander->getDistance(controlCenter, accessPoint, 3);
    TEST(std::cout << "real dist: " << dist << " for " << accessPoint << std::endl);
    */
    return -1;//dist;
}


/**
 * returns the length of this route
 */
real8 SpecialRoute::getRouteLength() const{
    return routeLength;
}


/**
 * returns the straight distance between the control center and access point
 */
real8 SpecialRoute::getSimpleRouteLength() const{
    TerrainBase::Loc ccLoc;
    controlCenter->get_center(ccLoc.x, ccLoc.y);
    return accessPoint.distance(ccLoc);
}


/**
 * returns whether the history record is active (still exists)
 */
bool SpecialRoute::mineralIsActive() const{
    return mineralRec->isActive();
}


/**
 * calls the commander to move the worker to its mineral
 */
void SpecialRoute::moveToMineral(GameObj *worker) const{
    commander->moveTo(worker, accessPoint);
}


/**
 * calls the commander to move the worker to its control center
 */
void SpecialRoute::moveToCC(GameObj *worker) const{
    commander->moveTo(worker, controlCenter);
}


/**
 * decrements the number of workers on this route
 */
void SpecialRoute::removeWorker(){ 
    currentWorkers--; 
}


/**
 * sets the id of this route
 */
void SpecialRoute::setID(sint4 id_){ 
    id = id_; 
}


/**
 * sets the length of this path
 */
void SpecialRoute:: setRouteLength(real8 length){
    routeLength = length;
}


/// decrements the number of path failures for this route
void SpecialRoute::planSuccessOccurred(){
    planFailures = 0;
}


/// increments the number of path failures for this route
void SpecialRoute::planFailureOccurred(){
    planFailures++;
}
